
/* Engine Alarm (version 6) for Arduino Leonardo
      Takes as inputs
        high water temperature switch
        low oil pressure switch
        IR temp measuring engine wet exhaust outlet to detect raw water pump fault ASAP

      In the event of a fault signal from any of the above, reports error in morse code
        W ( . - - ) for high water temperature
        O ( - - - ) for low oil pressure\
        X (- . . - ) for high wet exhaust temperature

        Uses LiqidCrystal Library for LCD (16 X 2)
        The circuit:
          LCD RS pin to digital pin 12
          LCD Enable pin to digital pin 11
          LCD D4 pin to digital pin 5
          LCD D5 pin to digital pin 4
          LCD D6 pin to digital pin 3
          LCD D7 pin to digital pin 2
          LCD R/W pin to ground
          10K pot:
            ends to +5V and ground
            wiper to LCD VO pin (pin 3)
*/

#include "IRTemp.h"
#include <LiquidCrystal.h>
#include "Timer.h"

//code speed defines
//the following based on http://www.kent-engineers.com/codespeed.htm
#define WPM 10          //code speed - change to speed up or slow down
#define DOT 1200/WPM    //element length in ms - DOT is one element long
#define DASH DOT*3      //DASH is three elements long
#define ELESPACE DOT    //space between individual dots and dashes in a letter
#define LTRSPACE DOT    //space between letters in a word
#define WRDSPACE DOT*7  //space between words

#define REPEATTIMES 3   //times to repeat morse code alarm

//other defines
#define BUZZ 0        //output for piezo buzzer
#define OBLED 13      //onboard LED
#define WATER 12      //water temperature switch input
#define OIL 11        //oil pressure switch input
#define XSETPOINT 150  //temperature at which exhaust alarm will sound
#define LOOPTIME  1500  //time between sensor checks



//IR Sensor Stuff
static const byte PIN_DATA    = 2; // Choose any pins you like for these
static const byte PIN_CLOCK   = 3;
static const byte PIN_ACQUIRE = 1;  //was 4
static const TempUnit SCALE = FAHRENHEIT; // Options are CELSIUS, FAHRENHEIT

char outString[16];   //for lcd messages
Timer t;
bool readIR = false;  //flag to enable ir sensor reads

int waterAlarm;
int oilAlarm;
int exTemp;
int foo;


/*************************************************************/

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

IRTemp irTemp(PIN_ACQUIRE, PIN_CLOCK, PIN_DATA);

void setup() {

  pinMode(BUZZ, OUTPUT);      //buzzer output pin
  pinMode(OBLED, OUTPUT);     //onboard LED
  pinMode(WATER, INPUT_PULLUP);      //water temp switch input - internal pullup resistor enabled
  pinMode(OIL, INPUT_PULLUP);        //oil pressure switch input - internal pullup resistor enabled


  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);

  // Print a message to the LCD.
  lcd.setCursor(1, 0);
  lcd.print("Legacy Engine");
  lcd.setCursor(4, 1);
  lcd.print("Monitor");

  delay(1000);              //leave message up a bit

  //set up LCD for running

  //clear display
  lcd.setCursor(0, 0);
  lcd.print("                ");
  lcd.setCursor(0, 1);
  lcd.print("                ");

  oilAlarm = digitalRead(OIL);
  updateOilDisp();
  waterAlarm = digitalRead(OIL);
  updateWaterDisp();
  exTemp = (int)irTemp.getIRTemperature(SCALE);
  updateExTempDisplay();

  //set up timer for ir temp reading interrupt
  t.every(2000, enableIR);    //every 2 seconds, enable a reading
}

void loop() {

  t.update();   //update timer library functions

//check oil switch
  
  foo = digitalRead(OIL);
  if (foo != oilAlarm){
    //things have changed, update display
    oilAlarm = foo;
    updateOilDisp();  
  }

//check water temperature switch
  
  foo = digitalRead(WATER);
  if (foo != waterAlarm){
    //things have changed, update display
    waterAlarm = foo;
    updateWaterDisp();  
  }
  
//check exhaust temperature if it's time to

  //is it time to check it?s
  if (readIR){
    readIR = false;   //set true in interrupt routine (enableIR) by timer interrupt
    foo = (int)irTemp.getIRTemperature(SCALE);
    if (foo != exTemp){
      //reading has changed, update display
      exTemp = foo;
      updateExTempDisplay();
    }
  }

//send morse code alarms if any
  
  if (waterAlarm == 0){
    //water alarm condition - send morse code
    for (int i = REPEATTIMES; i > 0; i--){
    sendW();
    delay(WRDSPACE);
    }
  }
  
  if (oilAlarm == 0){
    //oil alarm condition, send morse code
    for (int i  = REPEATTIMES; i > 0; i--){
    sendO();
    delay(WRDSPACE);
    }
  }
  
  if (exTemp > XSETPOINT){
    //exhaust temp alarm, send morse code
    for (int i = REPEATTIMES; i > 0; i--){
    sendX();
    delay(WRDSPACE);
    }
  }
  
}

/*********************************************
                 Functions

**********************************************/

void updateOilDisp(){
  lcd.setCursor(10, 1);
  if (!oilAlarm){
    lcd.print("OIL!!!");
  } else {
    lcd.print("OIL OK");
  }
}
  
void updateWaterDisp(){
  lcd.setCursor(0, 1);
  if (!waterAlarm){
    lcd.print("Water!!!");
  } else {
    lcd.print("Water OK");
  }
}
  
void updateExTempDisplay(){
  lcd.setCursor(0, 0);
  if (exTemp < XSETPOINT) {

    if (exTemp < 100) {
      lcd.print("Exhaust Temp  ");
      lcd.print(exTemp);
    } else {
      lcd.print("Exhaust Temp ");
      lcd.print(exTemp);
    }
  } else {     //temp was over setpoint - display !!! after label
    if (exTemp < 100) {
      lcd.print("Exhaust!!!    ");
      lcd.print(exTemp);
    } else {
      lcd.print("Exhaust!!!   ");
      lcd.print(exTemp);
    }
  }
}

void enableIR() {
  //called by timer interrupt so that ir sensor only gets
  //read at a time slightly longer than it's update time
  readIR = true;
}

void sendW() {
  // send a "w" to piezo buzzer ( . - - )
  dot();
  delay(ELESPACE);
  dash();
  delay(ELESPACE);
  dash();
  delay(LTRSPACE);
}

void sendO() {
  // send a "w" to piezo buzzer ( - - - )
  dash();
  delay(ELESPACE);
  dash();
  delay(ELESPACE);
  dash();
  delay(LTRSPACE);
}

void sendX() {
  // send a "w" to piezo buzzer ( - . . - )
  dash();
  delay(ELESPACE);
  dot();
  delay(ELESPACE);
  dot();
  delay(ELESPACE);
  dash();
  delay(LTRSPACE);
}
void dot() {
  digitalWrite(BUZZ, HIGH);     //turn on buzzer
  digitalWrite(OBLED, HIGH);    //turn on onboard led
  delay(DOT);
  digitalWrite(BUZZ, LOW);      //turn off buzzer
  digitalWrite(OBLED, LOW);     //turn off onboard LED
}

void dash() {
  digitalWrite(BUZZ, HIGH);     //turn on buzzer
  digitalWrite(OBLED, HIGH);    //turn on onboard led
  delay(DASH);
  digitalWrite(BUZZ, LOW);      //turn off buzzer
  digitalWrite(OBLED, LOW);     //turn off onboard LED
}




